# video_compression_model.py

import os
import cv2
import numpy as np
import tensorflow as tf
from featureExtraction import preprocess_frame
from globalVars import HEIGHT, LOGGER, NUM_COLOUR_CHANNELS, WIDTH


#from tensorflow.keras.mixed_precision import Policy

#policy = Policy('mixed_float16')
#tf.keras.mixed_precision.set_global_policy(policy)

def data_generator(videos, batch_size):
    # Infinite loop to keep generating batches
    while True:
        # Iterate over each video
        for video_details in videos:
            # Get the paths for compressed and original (uncompressed) video files
            video_path = os.path.join(os.path.dirname("test_data/validation/validation.json"), video_details["compressed_video_file"])
            uncompressed_video_path = os.path.join(os.path.dirname("test_data/validation/validation.json"), video_details["original_video_file"])
            
            # Open the video files
            cap_compressed = cv2.VideoCapture(video_path)
            cap_uncompressed = cv2.VideoCapture(uncompressed_video_path)
            
            # Lists to store the processed frames
            compressed_frame_batch = [] # Input data (Training)
            uncompressed_frame_batch = [] # Target data (Target)

            # Read and process frames from both videos
            while cap_compressed.isOpened() and cap_uncompressed.isOpened():
                ret_compressed, compressed_frame = cap_compressed.read()
                ret_uncompressed, uncompressed_frame = cap_uncompressed.read()
                if not ret_compressed or not ret_uncompressed:
                    break
                
                # Preprocess the compressed frame (input)
                compressed_frame = preprocess_frame(compressed_frame)
                
                # Preprocess the uncompressed frame (target)
                uncompressed_frame = preprocess_frame(uncompressed_frame) # Modify if different preprocessing is needed for target frames
                
                # Append processed frames to batches
                compressed_frame_batch.append(compressed_frame)
                uncompressed_frame_batch.append(uncompressed_frame)

                # If batch is complete, yield it
                if len(compressed_frame_batch) == batch_size:
                    yield (np.array(compressed_frame_batch), np.array(uncompressed_frame_batch)) # Yielding Training and Target data
                    compressed_frame_batch = []
                    uncompressed_frame_batch = []

            # Release video files
            cap_compressed.release()
            cap_uncompressed.release()

            # If there are frames left that don't fill a whole batch, send them anyway
            if len(compressed_frame_batch) > 0:
                yield (np.array(compressed_frame_batch), np.array(uncompressed_frame_batch))

class VideoCompressionModel(tf.keras.Model):
    def __init__(self):
        super(VideoCompressionModel, self).__init__()
        LOGGER.debug("Initializing VideoCompressionModel.")

        # Input shape (includes channels for edges and histogram)
        input_shape_with_histogram = (HEIGHT, WIDTH, NUM_COLOUR_CHANNELS)

        # Encoder part of the model
        self.encoder = tf.keras.Sequential([
            tf.keras.layers.InputLayer(input_shape=input_shape_with_histogram),
            tf.keras.layers.Conv2D(64, (3, 3), activation='relu', padding='same'),
            tf.keras.layers.MaxPooling2D((2, 2), padding='same'),
            tf.keras.layers.Conv2D(32, (3, 3), activation='relu', padding='same'),
            tf.keras.layers.MaxPooling2D((2, 2), padding='same')
        ])

        # Decoder part of the model
        self.decoder = tf.keras.Sequential([
            tf.keras.layers.Conv2DTranspose(32, (3, 3), activation='relu', padding='same'),
            tf.keras.layers.UpSampling2D((2, 2)),
            tf.keras.layers.Conv2DTranspose(64, (3, 3), activation='relu', padding='same'),
            tf.keras.layers.UpSampling2D((2, 2)),
            tf.keras.layers.Conv2DTranspose(1, (3, 3), activation='sigmoid', padding='same')
        ])

    def call(self, inputs):
        # Encode the input
        encoded = self.encoder(inputs)
        # Decode the encoded representation
        decoded = self.decoder(encoded)
        return decoded
